package io.tswf.gpt.mistral.client

import feign.okhttp.OkHttpClient
import groovy.transform.CompileStatic
import groovy.transform.Memoized
import io.tswf.gpt.mistral.api.ApiClient
import io.tswf.gpt.mistral.api.client.DefaultApi
import io.tswf.groovy.bettergroovy.transforms.nullsafe.NullSafe
import okhttp3.Credentials

@CompileStatic
class MistralApi {

    @NullSafe
    String mistralApiToken

    @NullSafe
    String mistralBaseUrl = System.findGlobalProperty("mistral.api.base-url")
            .orElse(MistralURLs.MISTRAL_DEFAULT_BASE_PATH_V1)

    Proxy mistralProxy = null

    String mistralProxyUser = null

    String mistralProxyPassword = null

    MistralApi(@NullSafe String mistralApiToken, @DelegatesTo(MistralApi) Closure configurator = {  }) {
        this.mistralApiToken = mistralApiToken
        if (configurator != null) {
            configurator.delegate = this
            configurator.call(this)
        }
    }

    @Memoized
    DefaultApi getDefaultApi() {
        return apiClientsBuilder.buildClient(DefaultApi)
    }

    @Memoized
    ApiClient getApiClientsBuilder() {
        return new ApiClient()
                .tap {
                    feignBuilder.tap {

                        // Гугловая авторизация на API
                        requestInterceptor { request ->
                            // API ключ Mistral
                            request.header("Authorization", "Bearer $mistralApiToken")
                        }

                        client(
                                new OkHttpClient(
                                        new okhttp3.OkHttpClient.Builder()
                                                .tap {
                                                    mistralProxy?.let {
                                                        proxy(it)
                                                        // Авторизация для прокси, если требуется
                                                        if (mistralProxyUser.isNotNullOrBlank()) {
                                                            if (mistralProxyPassword == null) {
                                                                throw new IllegalStateException("Password was not given for proxy user with username: '${mistralProxyUser}'")
                                                            }

                                                            if (mistralProxy == null) {
                                                                throw new IllegalStateException("Proxy password set when proxy is null!")
                                                            }

                                                            if (mistralProxy.type() != Proxy.Type.HTTP) {
                                                                throw new IllegalStateException("Proxy authorization is only supported for HTTP proxy type, not for '${mistralProxy.type()?.name()}'!")
                                                            }

                                                            proxyAuthenticator { route, response ->
                                                                response.request().newBuilder()
                                                                        .header("Proxy-Authorization", Credentials.basic(mistralProxyUser, mistralProxyPassword))
                                                                        .build()
                                                            }
                                                        }
                                                    }
                                                }
                                                .build()
                                )
                        )
                    }

                    basePath = mistralBaseUrl
                }
    }
}
